package org.modeshape.test;

import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.Asynchronous;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.jcr.nodetype.NodeType;
import javax.transaction.UserTransaction;

import org.jboss.logging.Logger;

@Stateless
@LocalBean
@TransactionManagement(TransactionManagementType.BEAN)
public class BulkLoadTest {
	private static Logger log = Logger.getLogger(BulkLoadTest.class);
	
	public class Config {
		int nodeCount;
		int propCount;
		
		public Config(int nodeCount, int propCount) {
			this.nodeCount = nodeCount;
			this.propCount = propCount;
		}
	}
	
	@Resource(mappedName="java:/jcr/sample")
	private Repository repository;
	
	@Resource
	private UserTransaction ut;
	
	private Session session;
	private List<Config> configs = Arrays.asList(
				new Config(100, 10),
				new Config(100, 10),
				new Config(100, 50));
	
	private int nodeCount = 0;
	private int propCount = 0;
	
	@Asynchronous
	public void run() throws Exception {
		createNodes();
	}
	
	private void createNodes() throws Exception {
		nodeCount = 0;
		propCount = 0;
		
		log.info("Creating nodes");
		ut.begin();
		try {
			session = repository.login(new SimpleCredentials("admin", "admin".toCharArray()));
			Node node = session.getRootNode().addNode("test", NodeType.NT_UNSTRUCTURED);
			try {
				createNodes(node, 0);
				session.save();
			} finally {
				session.logout();
			}
			ut.commit();
		} catch(Exception e) {
			log.error("Error creating nodes", e);
			ut.rollback();
		} 
		log.info("Created a total of " + nodeCount + " nodes, " + propCount + " properties");
	}
	
	private void createNodes(Node parent, int level) throws Exception {
		for (int i = 1; i <= configs.get(level).nodeCount; i++) {
			Node node = parent.addNode("node" + i, NodeType.NT_UNSTRUCTURED);
			for (int j = 1; j <= configs.get(level).propCount; j++) {
				node.setProperty("prop" + j, node.getIdentifier() + "-prop" + j);
				propCount++;
			}
			nodeCount++;
			if ((nodeCount % 1000) == 0) {
				log.info("Created " + nodeCount + " nodes, " + propCount + " properties - saving Session and committing transaction");
				session.save();
				ut.commit();
				ut.begin();
			}
			if (level < (configs.size() - 1)) {
				createNodes(node, level + 1);
			}
		}
	}

}
